@@ -1,12 +1,15 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# Copyright (C) 2023 Fortra. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
+
 import array
 
-from ImpactPacket import Header, ImpactPacketException, PacketBuffer
+from impacket.ImpactPacket import Header, ImpactPacketException, PacketBuffer
 
 class IP6_Extension_Header(Header):
 # --------------------------------- - - - - - - -
@@ -40,7 +43,7 @@ class IP6_Extension_Header(Header):
         for option in self._option_list:
             option_str = str(option)
             option_str = option_str.split('\n')
-            option_str = map(lambda s: (' ' * 4) + s, option_str)
+            option_str = [(' ' * 4) + s for s in option_str]
             s += '\n'.join(option_str) + '\n'
         
         return s
@@ -53,7 +56,7 @@ class IP6_Extension_Header(Header):
 
         buffer = array.array('B', buffer[self.get_headers_field_size():])
         if remaining_bytes > len(buffer):
-            raise ImpactPacketException, "Cannot load options from truncated packet"
+            raise ImpactPacketException("Cannot load options from truncated packet")
 
         while remaining_bytes > 0:
             option_type = buffer[0]
@@ -136,7 +139,7 @@ class IP6_Extension_Header(Header):
         data = self.get_data_as_string()
 
         # Update the header length
-        self.set_header_extension_length(self.get_header_size() / 8 - 1)
+        self.set_header_extension_length(self.get_header_size() // 8 - 1)
 
         # Build the entire extension header packet
         header_bytes = self.get_buffer_as_string()
@@ -164,7 +167,7 @@ class Extension_Option(PacketBuffer):
 
     def __init__(self, option_type, size):
         if size > Extension_Option.MAX_OPTION_LEN:
-            raise ImpactPacketException, "Option size of % is greater than the maximum of %d" % (size, Extension_Option.MAX_OPTION_LEN)
+            raise ImpactPacketException("Option size of % is greater than the maximum of %d" % (size, Extension_Option.MAX_OPTION_LEN))
         PacketBuffer.__init__(self, size)
         self.set_option_type(option_type)
 
@@ -217,10 +220,10 @@ class Option_PADN(Extension_Option):
 
     def __init__(self, padding_size):
         if padding_size < 2:
-            raise ImpactPacketException, "PadN Extension Option must be greater than 2 bytes"
+            raise ImpactPacketException("PadN Extension Option must be greater than 2 bytes")
 
         Extension_Option.__init__(self, Option_PADN.OPTION_TYPE_VALUE, padding_size)
-        self.set_data('\x00' * (padding_size - 2))
+        self.set_data(b'\x00' * (padding_size - 2))
 
 class Basic_Extension_Header(IP6_Extension_Header):
     MAX_OPTIONS_LEN = 256 * 8
@@ -266,7 +269,7 @@ class Hop_By_Hop(Basic_Extension_Header):
     
     @classmethod
     def get_decoder(self):
-        import ImpactDecoder
+        from impacket import ImpactDecoder
         return ImpactDecoder.HopByHopDecoder
 
 class Destination_Options(Basic_Extension_Header):
@@ -275,7 +278,7 @@ class Destination_Options(Basic_Extension_Header):
     
     @classmethod
     def get_decoder(self):
-        import ImpactDecoder
+        from impacket import ImpactDecoder
         return ImpactDecoder.DestinationOptionsDecoder
 
 class Routing_Options(IP6_Extension_Header):
@@ -307,7 +310,7 @@ class Routing_Options(IP6_Extension_Header):
         
     @classmethod
     def get_decoder(self):
-        import ImpactDecoder
+        from . import ImpactDecoder
         return ImpactDecoder.RoutingOptionsDecoder
 
     def get_headers_field_size(self):
