@@ -1,6 +1,6 @@
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2018 SecureAuth Corporation. All rights reserved.
+# Copyright (C) 2023 Fortra. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -581,10 +581,11 @@ class ESENT_PAGE:
         if self.__DBHeader['Version'] == 0x620 and self.__DBHeader['FileFormatRevision'] >= 17 and self.__DBHeader['PageSize'] > 8192:
             valueSize = unpack('<H', tag[:2])[0] & 0x7fff
             valueOffset = unpack('<H',tag[2:])[0] & 0x7fff
-            tmpData = list(self.data[baseOffset+valueOffset:][:valueSize])
-            pageFlags = ord(tmpData[1]) >> 5
-            tmpData[1] = chr(ord(tmpData[1:2]) & 0x1f)
-            tagData = "".join(tmpData)
+            tmpData = bytearray(self.data[baseOffset+valueOffset:][:valueSize])
+            pageFlags = tmpData[1] >> 5
+            tmpData[1] = tmpData[1:2][0] & 0x1f
+            tmpData = bytes(tmpData)
+            tagData = tmpData
         else:
             valueSize = unpack('<H', tag[:2])[0] & 0x1fff
             pageFlags = (unpack('<H', tag[2:])[0] & 0xe000) >> 13
